@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, 
- * Boston, MA 02111-1307, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __VINAGRE_TAB_H__
@@ -78,6 +76,8 @@ void		  vinagre_tab_set_notebook	(VinagreTab *tab,
 VinagreNotebook   *vinagre_tab_get_notebook	(VinagreTab *tab);
 
 void		  vinagre_tab_take_screenshot	(VinagreTab *tab);
+void		  vinagre_tab_paste_text	(VinagreTab *tab,
+						 const gchar *text);
 G_END_DECLS
 
 #endif  /* __VINAGRE_TAB_H__  */
