@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, 
- * Boston, MA 02111-1307, USA.
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <string.h>
@@ -142,3 +140,24 @@ vinagre_utils_escape_underscores (const gchar* text,
 
 	return g_string_free (str, FALSE);
 }
+
+void
+vinagre_utils_show_many_errors (const gchar *message, GSList *items, GtkWindow *parent)
+{
+  GString *msg;
+  GSList  *l;
+
+  msg = g_string_new (message);
+  g_string_append_c (msg, '\n');
+
+  for (l = items; l; l = l->next)
+    {
+      g_string_append_printf (msg, "\n%s", (gchar *)l->data);
+      g_free (l->data);
+    }
+
+    vinagre_utils_show_error (msg->str, parent);
+    g_slist_free (items);
+    g_string_free (msg, TRUE);
+}
+/* vim: ts=8 */
